'use strict';

const DEFAULT_SHUTDOWN_DELAY = 5000;

// ** Dependencies
const EventEmitter = require('events').EventEmitter;
const logger = require('./logging').createLogger('program');

// Program Events
const events = new EventEmitter();

function shutdown(delay) {

    delay = delay || DEFAULT_SHUTDOWN_DELAY;

    logger.debug('Shutting down program...');
    events.emit('shutdown');

    setTimeout(() => {
        logger.debug('The program did not shutdown in time forcing shutdown...', {
            delay: delay * .001 + ' seconds'
        });

        process.exit();
    }, delay ).unref();
}

// Add process hooks
process.on('SIGINT', function () {
    logger.debug('**** RECEIVED SIGNAL [SIGINT] ****');
    shutdown()
});

process.on('SIGTERM', function () {
    logger.debug('**** RECEIVED SIGNAL [SIGTERM] ****');
    shutdown()
});

// ** Exports
module.exports.on = events.on.bind(events);
module.exports.shutdown = shutdown;